/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#ifndef _CASP_BSP_SPI_H_
#define _CASP_BSP_SPI_H_

#include "sam_hwd.h"

#define SPI_BASE_CLK                    16000000UL
#define SPI_DUMMY_BYTE                  0x00//0xff//casp

#define SPI_MODE0			0x00
#define SPI_MODE1			0x01
#define SPI_MODE2			0x02
#define SPI_MODE3			0x03

class SPISettings
{
public:
    SPISettings(const _UINT32 clock=1000000, const _BYTE bitOrder=MSBFIRST, const _BYTE dataMode=SPI_MODE0) :_clock(clock), _bitOrder(bitOrder), _dataMode(dataMode) {}
    const _UINT32 _clock;
    const _BYTE  _bitOrder;
    const _BYTE  _dataMode;
};

class SpiComm
{
public:
    SpiComm(const _UINT32 spi_base_clk=SPI_BASE_CLK);
    ~SpiComm();
    void begin();
    void end();
    void setClockDivider(const unsigned short);
    void setBitOrder(const unsigned char);
    void beginTransaction(const SPISettings &settings=SPISettings());
    void endTransaction();
    void send(const unsigned char);
    unsigned char recv();
    unsigned char transfer(const unsigned char);
    unsigned short transfer16(const unsigned short);
private:
    inline void wait(){for(volatile _WORD i=0; i<m_clk_div; ++i);}
private:
    _WORD m_clk_div;
    const _UINT32 m_spi_base_clk;
    _BYTE m_msb_first;
};
extern SpiComm SPI;

#endif//_CASP_BSP_SPI_H_
