/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#ifndef _SAM_HWD_H_
#define _SAM_HWD_H_

#include "sim_common_definitions.h"
#include "sam_hwd_defs.h"
#include "sim_debug.h"
#include <stdlib.h>

#ifdef __cplusplus
extern "C"{
#endif

#define HIGH                    1
#define LOW                     0
#define INPUT                   0
#define OUTPUT                  1
#define INPUT_PULLUP            2
#define INPUT_PULLDOWN          3
#define OUTPUT_OPENDRAIN        4
#define INPUT_DISABLE           5
#define LSBFIRST                0
#define MSBFIRST                1
#define CHANGE                  4
#define FALLING                 2
#define RISING                  3

#define GPIO_ADC_COUNT		12
#define GPIO_PWM_COUNT		20
#define GPIO_DAC_COUNT		2
#define GPIO_DIO_COUNT		36

#define GPIO_MAX_EXTI_SIZE	8

//
#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))
#define radians(deg) ((deg)*DEG_TO_RAD)
#define degrees(rad) ((rad)*RAD_TO_DEG)
#define sq(x) ((x)*(x))

#define lowByte(w) ((_BYTE) ((w) & 0xff))
#define highByte(w) ((_BYTE) ((w) >> 8))

#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))

//CASP HAL API functions
void setup();
void loop();
void analogReadResolution(unsigned char res);//sets ADC resolution, not used
void analogWriteResolution(unsigned char res);//sets resolution for all PWM outputs
void analogReadResolutionPin(unsigned char res, unsigned char pin);//set ADC resolution for a pin
void analogWriteResolutionPin(unsigned char res, unsigned char pin);//set resolution for a PWM output pin
void pinMode(unsigned char pin, unsigned char mode);
void digitalWrite(unsigned char pin, unsigned char val);
int digitalRead(unsigned char pin);
int analogRead(unsigned char pin);
void analogWrite(unsigned char pin, int val);
unsigned long long millis();
unsigned long long micros();
void delay(unsigned long delay);
void delayMicroseconds(unsigned long delay);
int digitalPinToInterrupt(int pin);
void attachInterrupt(int intr_id, void (*function)(void), unsigned int mode);

//internal functions
void init_before_setup();
void init_after_setup();
void init_variant_before_setup();
void init_variant_after_setup();
void loop_system(void);
//
void initADC();
void startADC();
void updateADC();
void initPWM();
void startPWM();
void initDIO();
///////////////////////////serial ports///////////////////////////////////
#define SERIAL_TX_BUFFER_SIZE	64
#define SERIAL_RX_BUFFER_SIZE	64
//
#define SERIAL_8N1				0x01
#define SERIAL_8E1				0x02
#define SERIAL_8O1				0x03
//
class SerialComm
{
public:
    SerialComm();
    ~SerialComm();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(unsigned char*, int);
    inline int readBytes(char *buffer, int length) {return readBytes((unsigned char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush();
private:
};
extern SerialComm Serial;
////////////////////////////////////////////////////////////////////////////
//This function is executed in case of error occurrence
void _Error_Handler(const char *, int);
//#define Error_Handler() _Error_Handler(__FILE__, __LINE__)
////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
}
#endif

#endif //_SAM_HWD_H_
